
/*
 Here is an example of how to work with the PLU movement data and
 index files from a STANDALONE program (i.e. not within user exits).
 This same logic will work with other Retail/DOS index files.
 The key length for this example is assumed to be 6.
*/

//--------------------------------------------------------

#include <stdlib.h>
#include <string.h>
#include "bizlib.h"
#include "movement.h"
#include "header.h"

#define KEY_LENGTH 6

struct MOVEMENT mv;
FILE *fptr_mv;
struct FHD *hdr;
int bt_status, midx;
char key[20];
long rp;

void put_err_msg(char *);
void error_message(char *, char *, int);

void main()
{
    // initialize internal Retail/DOS file routines

    strcpy(err.main_id,"SAMPLE.EXE"); // Your program name goes here
    fio_init(NOABORT);
    bt_init();

    // create data and index files if they do not already exist

    if ((access("MOVEMENT.DAT",0)) == -1) {
        memset(&mv,aNUL,sizeof(mv));
        hdr = (struct FHD *)&mv;
        hdr->nextrec = 1;
        hdr->lastrec = 0;
        hdr->usedrec = 0;
    	fptr_mv = fio_openfile("MOVEMENT.DAT","wb",DENYNONE,FORCE);
        fio_access(fptr_mv,0L,sizeof(mv),&mv,sizeof(mv),WRITE,FORCE);
        fio_closefile(fptr_mv);
        printf("MOVEMENT.DAT created\n");
    }

    if ((access("MOVEMENT.IX1",0)) == -1) {
        bt_status = bt_create("MOVEMENT.IX1",KEY_LENGTH,UNIQUE);
        printf("MOVEMENT.IX1 created\n");
    }

    // open data and index files

    fptr_mv = fio_openfile("MOVEMENT.DAT","rb+",DENYNONE,FORCE);
    if (fio_status) {
        exit(9);
    }

    midx = bt_open("MOVEMENT.IX1",FLUSH);
    if (midx < 0) {
        exit(9);
    }

    // example to read specific key PLU 000005 from movement file
    // if not found then create new movement record for PLU 000005

    strcpy(key,"000005");
    bt_status = bt_search(midx,key,&rp);
    if (!bt_status) {
        fio_access(fptr_mv,rp,sizeof(mv),&mv,sizeof(mv),READ,FORCE);
        printf("plu <%s> qty = <%li>\n",key,mv.qty_onhand);
    } else {
        rp = fio_getrec(fptr_mv,sizeof(mv));
        if (!fio_status) {
            lock(fptr_mv,rp,sizeof(mv));
            memset(&mv,0,sizeof(mv));
            strcpy(mv.item,key);
            fio_access(fptr_mv,rp,sizeof(mv),&mv,sizeof(mv),WRITE,FORCE);
            if (!fio_status) {
                 bt_status = bt_insert(midx,key,rp);
                 if (bt_status && bt_status != DUPLICATE_KEY) {
                     bt_error(bt_status);
                     exit(9);
                 }
            }
            unlock(fptr_mv,rp,sizeof(mv));
        }
        printf("plu <%s> movement record created\n",key);
    }

    // example loop through movement file for all PLUs

    // get offset for first movement record

    bt_status = bt_firstkey(midx,&rp);

    while (1) {
        if (bt_status) {
            if (bt_status != EMPTY_ROOT && bt_status != KEY_NOT_FOUND &&
                                           bt_status != END_OF_FILE) {
                bt_error(bt_status);
                exit(9);
            } else {
                break;
            }
        }

        fio_access(fptr_mv,rp,sizeof(mv),&mv,sizeof(mv),READ,FORCE);

        printf("plu <%s> qty = <%li>\n",mv.item,mv.qty_onhand);

        // get offset for next movement record
        bt_status = bt_searchnext(midx,&rp);
    }

    // delete PLU 000005 from movement file

    strcpy(key,"000005");
    bt_status = bt_search(midx,key,&rp);
    if (!bt_status) {
        bt_status = bt_delete(midx,key,rp);
        if (bt_status) {
            bt_error(bt_status);
            exit(9);
        }
        fio_delrec(fptr_mv,sizeof(mv),rp);
        printf("plu <%s> deleted\n",key);
    }

    // close index and data files

    bt_close(midx);
    fio_closefile(fptr_mv);

	// close all index and data files if needed

	bt_closeall();
	fio_closeall();
}

// This function is not needed if you are linking your program with
// OPFUNC.OBJ or if you are performing user exit coding.
void put_err_msg(char *s)
{
	error_message(s,"PRESS <ENTER> TO CONTINUE",1);
}

// This function is not needed if you are linking your program with
// OPFUNC.OBJ or if you are performing user exit coding.
void error_message(char *s1, char *s2, int w)
{
	mvcur(0,0);
    printf("                                               ");
	mvcur(1,0);
    printf("                                               ");
	mvcur(0,0);
    printf("%s",s1);
	mvcur(1,0);
    printf("%s",s2);
	if (w) {
        getch();
	}
}

//------------------------------------------------------------

/*
 If you are going to need something like this in a user exit then
 you will only want to use the portion of the code that performs
 the reading of the files (i.e. the file has probably already been
 opened by the register program). The file and index pointer will
 usually be available in the global structure. There are instances
 where the the file or index pointer might be local to an internal
 Retail/DOS function. If this is the case then accessing the open
 file will be very dificult.
 There are some files that do not stay open all the time in the
 register program. These files are opened when needed and
 then closed. You will have to play around to determine if
 the specific file you want to access is already open.
 You will know this because fio_openfile() will return an error
 in the global variable fio_status if the file is already open.
*/

